# genetic simulation
setwd("/users/mcvean/xilin/xilin/Multimorbidity-biobank/")
source("topic_functions.R")
source("simulation_functions.R")
args <- commandArgs(trailingOnly = TRUE)
v_id <- as.integer(args[1])
s_id <- as.integer(args[2])
set.seed(19940110)

# for the v_id = 2 case, perform 50 reps 
rep_number <- 10
# 1. SNP->T->D
if(v_id == 1){
  cont_v2t <- 2*(1:10)[s_id] # number of SNP2topic associations assumed in inference
}else{
  cont_v2t <- 20 
}
# 2. causal disease SNP->D->T
if(v_id == 2){
  disease2topic <- (1:10/20)[s_id]
  rep_number <- 50 
}else{
  disease2topic <- 0.1
}
# 3. SNP*T->D
if(v_id == 3){
  itr_effect <- (1:10/2.5)[s_id] # interaction effect
}else{
  itr_effect <- 2
}
# 4. SNP->D; SNP->T->D
if(v_id == 4){
  topic2disease <- (1:10/2.5)[s_id] # topic to disease effect
}else{
  topic2disease <- 2
}

disease_number <- 20

genetic_simulation <- list()
for(rep_id in 1:rep_number){
  print(paste0("V: ", v_id, "; S: ", s_id, "; repitetion: ", rep_id))
  ###########################################
  # Part 1:
  # simulation of topic: conditional on SNP, causal disease 
  ###########################################
  
  rslts <- simulate_topics(topic_number = 2, disease2topic  = disease2topic, v2t = cont_v2t )
  para_sim <- rslts[[1]]
  genetics_population <- rslts[[2]]
  causal_disease <- rslts[[3]]
  ###########################################
  # Part 2:
  # simulation diseases generated by topic 
  ###########################################
  reslt_ds <- simulate_genetic_disease_from_topic(para_sim, genetics_population,causal_disease,
                                                  disease_number = disease_number, itr_effect = itr_effect,
                                                  topic2disease = topic2disease, v2t = cont_v2t)
  rec_data <- reslt_ds[[1]]   
  ds_list <- reslt_ds[[2]]  
  interact_disease <- reslt_ds[[3]] 
  pleiotropp_disease <- reslt_ds[[4]] 
  # perform inference
  para <- inf_genetic_simulation(rec_data, ds_list, topic_number = 2)
  #######################
  # save the data
  #######################
  # compute patient loading
  patient_loadings <- sweep((para$alpha_z - 1), 1, rowSums(para$alpha_z -1), FUN="/")
  
  # compute disease matrix
  disease_matrix <- rec_data %>%
    select(-age_diag) %>%
    mutate(disease = 1, diag_icd10 = factor(diag_icd10, levels = 1:(max(rec_data$diag_icd10)))) %>%
    pivot_wider(names_from = diag_icd10, values_from = disease,values_fill = 0, names_sort = T) %>%
    select(-eid) %>%
    as.matrix()
  
  # simulation paramters
  sim_para <- list()
  sim_para$cont_v2t <-cont_v2t
  sim_para$disease2topic <- disease2topic
  sim_para$itr_effect <- itr_effect
  sim_para$topic2disease <- topic2disease
  sim_para$disease_number <- disease_number
  genetic_simulation[[rep_id]] <- list(genetics_population, patient_loadings, disease_matrix, sim_para)
  
}

save(genetic_simulation, file = paste0("/users/mcvean/xilin/xilin/Multimorbidity-biobank/simulations/genetic_simulation/","gen_simulation_v",v_id,"s",s_id,".RData"))


