test_that("test lfa inference", {
  para <- topic_init_lfa_cvb(rec_data = HES_age_example, ds_list=UKB_349_disease, topic_num=10)
  # update the zn; each step should increase the lower bound
  lb1 <- CVB_lb_lfa(para)
  para <- CVB0_E_zn(para)
  lb2 <- CVB_lb_lfa(para)
  expect_gt(lb2, lb1)
  # update the beta
  para <- update_beta_lfa(para)
  lb3 <- CVB_lb_lfa(para)
  expect_gt(lb3, lb2)
})

test_that("lfa accuracy", {
  # simulate some data using Yidong's code
  # Parameters:
  K <- 4    # number of topics
  S <- 15     # total number of diseases (internal and terminal)
  D <- 5000   # number of individuals in the training data
  alpha <- rep(0.1,4)    # Dirichlet prior for topic weights


  # manually set the topics
  topics <- c( c(0.9, 0.9, 0.9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
               c(0, 0, 0, 0.9, 0.9, 0.9, 0, 0, 0, 0, 0, 0, 0, 0, 0),
               c(0, 0, 0, 0, 0, 0, 0.9, 0.9, 0.9, 0, 0, 0, 0, 0, 0),
               c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9) )

  topics <- matrix(topics,nrow=K,byrow=TRUE)/3

  theta <- gtools::rdirichlet(D, alpha)


  # Simulate input data: training data
  for ( ds in 1:1 ){

    # Create directory to save the input dataset:
    setwd("/Users/ydzhang/Desktop/")
    dir <- as.character(ds)
    dir.create( dir )
    setwd( dir )


    # Simulate topic dist for each indiv:
    topic_dist <- topic_dist_simu( alpha, D, K )
    # Write data:
    fwrite(topic_dist,"topics.dist.txt")


    # Simulate topic assignment for each word in each indiv:
    topic_assign <- topic_assign_simu( topic_dist, D, K, S )
    # Write data:
    fwrite(topic_assign,"topics.assignment.txt")


    # Simulate disease states for all indivs:
    disease_state <- disease_simu_nonparallel( topic_assign, topics )

    # Save simulated data:
    input.name <- paste("D", ds, "LDA.input.txt", sep=".")
    fwrite(disease_state,input.name)

  }




  para <- topic_init_lfa_cvb(rec_data = HES_age_example, ds_list=UKB_349_disease, topic_num=10)
  # update the zn; each step should increase the lower bound
  lb1 <- CVB_lb_lfa(para)
  para <- CVB0_E_zn(para)
  lb2 <- CVB_lb_lfa(para)
  expect_gt(lb2, lb1)
  # update the beta
  para <- update_beta_lfa(para)
  lb3 <- CVB_lb_lfa(para)
  expect_gt(lb3, lb2)
})
